package com.shop.controller;
import com.shop.annotation.IsLogin;
import com.shop.base.BaseController;
import com.shop.constant.Constant;
import com.shop.constant.MessageModel;
import com.shop.gto.GoodsDto;
import com.shop.model.*;
import com.shop.service.GoodsService;
import com.shop.util.MessageModelUtil;
import com.shop.vo.LoginIdentity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import javax.servlet.http.HttpSession;
import java.util.List;
import java.util.Map;

/**
 * 后台登录.....
 *商家后台登录
 * @author DY
 * @create 2018-03-06 12:03
 **/
@RestController
@RequestMapping("goods")
public class GoodsController extends BaseController {

    @Autowired
    private GoodsService goodsService;
    /**
     *@author DY
     *@create 2018/3/26 17:28
     *test
     */
    @RequestMapping("test")
    public MessageModel test(Goods goods) throws Exception {
         goodsService.insertSize( goods);
        return MessageModelUtil.getSuccessMessageModel();
    }

    /**
     *@author DY
     *@create 2018/3/26 17:28
     *根据选择属性查询商品
     */
    @IsLogin
    @RequestMapping("query_goods_by_attribute")
    public MessageModel query_goods_by_attribute(String size,String color,Boolean isCustom,Integer goods) throws Exception {
        MessageModel messageModel = goodsService.query_goods_by_attribute(size,color,isCustom,goods);
        return messageModel;
    }


    /**
     *@author DY
     *@create 2018/3/26 17:28
     *查询尺码详情表(用于自定义添加下拉框选择)
     */
    @IsLogin
    @RequestMapping("query_size")
    public MessageModel query_size_add() throws Exception {
        MessageModel messageModel=goodsService.query_size_add();
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/26 17:28
    *查询可定制项
    */
    @IsLogin
    @RequestMapping("query_custom_names")
    public MessageModel query_custom_names() throws Exception {
        MessageModel messageModel = goodsService.query_custom_names();
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/26 17:28
    *查询场合
    */
    @IsLogin
    @RequestMapping("query_product_scene")
    public MessageModel query_product_scene(Integer gender) throws Exception {
        MessageModel messageModel=goodsService.query_product_scene( gender);
        return messageModel;
    }
    /**
    *@author DY
    *@create 2018/3/26 17:28
    *查询二级分类(根据男女)
    */
    @IsLogin
    @RequestMapping("query_product_category_one")
    public MessageModel query_product_category(Integer gender) throws Exception {
        MessageModel messageModel=goodsService.query_product_category( gender);
        return messageModel;
    }

    /**
     *@author DY
     *@create 2018/3/26 17:28
     *查询三级分类(根据二级目录)
     */
    @IsLogin
    @RequestMapping("query_product_category_two")
    public MessageModel query_product_category_two(Integer categoryId) throws Exception {
        MessageModel messageModel=goodsService.query_product_category_two( categoryId);
        return messageModel;
    }




    /**
     *@author DY
     *@create 2018/3/26 17:28
     *查询尺码详情表(根据三级目录)
     */
    @IsLogin
    @RequestMapping("query_size_by_three")
    public MessageModel query_size(Integer categoryIdChildren) throws Exception {
        MessageModel messageModel=goodsService.query_size( categoryIdChildren);
        return messageModel;
    }


    /**
    *@author DY
    *@create 2018/3/6 12:07
    *添加商品
    */
    @IsLogin
    @RequestMapping("add_goods")
    public MessageModel add_goods(Goods goods, HttpSession session) throws Exception {
        LoginIdentity loginIdentity = (LoginIdentity)session.getAttribute(Constant.LOGIN_USER_KEY);
        MessageModel messageModel=goodsService.add_goods(goods,loginIdentity);
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/21 10:43
    *商品总览接口
    */
    @IsLogin
    @RequestMapping("goods_overview")
    public MessageModel goods_overview(HttpSession session) throws Exception {
        LoginIdentity loginIdentity = (LoginIdentity) session.getAttribute(Constant.LOGIN_USER_KEY);
        Map map=goodsService.goods_overview (loginIdentity);
        MessageModel messageModel=MessageModelUtil.getSuccessMessageModel(map);
        return messageModel;
    }

    /**
     *@author DY
     *@create 2018/3/21 10:43
     *商品列表接口
     */
    @IsLogin
    @RequestMapping("goods_list_product")
    public MessageModel goods_list(GoodsDto goodsDto, HttpSession session) throws Exception {
        LoginIdentity loginIdentity = (LoginIdentity)session.getAttribute(Constant.LOGIN_USER_KEY);
        MessageModel messageModel=goodsService.goods_list(goodsDto,loginIdentity);
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/23 11:34
    *查看商品审核详情
    */
    @IsLogin
    @RequestMapping("query_goods_check_status")
    public MessageModel query_goods_check_status(Integer id) throws Exception {
        MessageModel messageModel=goodsService.query_goods_check_status(id);
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/23 12:05
    *查看商品详情
    */
    @IsLogin
    @RequestMapping("query_goods_detail")
    public MessageModel query_goods_detail(Integer id) throws Exception {
        Map map=goodsService.query_goods_detail(id);
        MessageModel messageModel= MessageModelUtil.getSuccessMessageModel(map);
        return messageModel;
    }


    /**
    *@author DY
    *@create 2018/3/23 12:05
    *查看商品库存
    */
    @IsLogin
    @RequestMapping("query_goods_stock")
    public MessageModel query_goods_stock(Integer id) throws Exception {
        List<ProductNoCustom> map=goodsService.query_goods_stock(id);
        MessageModel messageModel= MessageModelUtil.getSuccessMessageModel(map);
        return messageModel;
    }


    /**
    *@author DY
    *@create 2018/3/23 12:05
    *查看具体商品详情
    */
    @IsLogin
    @RequestMapping("query_product_detail")
    public MessageModel query_product_detail(Integer id) throws Exception {
        MessageModel messageModel= goodsService.query_product_detail(id);;
        return messageModel;
    }


    /**
    *@author DY
    *@create 2018/3/26 17:37
    *商品集体上下架,设置为新品/推荐
    */
    @IsLogin
    @RequestMapping("is_marketable_is_new_product_recommend")
    public MessageModel is_marketable_is_new_product_recommend(IsMarketableNewRecommend isMarketableNewRecommend) throws Exception {
        MessageModel messageModel=goodsService.is_marketable_is_new_product_recommend(isMarketableNewRecommend);
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/26 18:34
    *货品编辑
    */
    @IsLogin
    @RequestMapping("edit_product")
    public MessageModel edit_product(Goods goods) throws Exception {
        MessageModel messageModel=goodsService.edit_product(goods);
        return messageModel;
    }


    /**
    *@author DY
    *@create 2018/3/26 18:34
    *货品编辑_修改默认商品
    */
    @IsLogin
    @RequestMapping("edit_default_product")
    public MessageModel edit_default_product(Integer id,String defaultGoods) throws Exception {
        MessageModel messageModel=goodsService.edit_default_product(id,defaultGoods);
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/27 16:06
    *货品删除
    */
    @IsLogin
    @RequestMapping("delete_goods")
    public MessageModel delete_goods(String ids) throws Exception {
        MessageModel messageModel=goodsService.delete_goods(ids);
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/27 16:06
    *商品添加
    */
    @IsLogin
    @RequestMapping("add_product")
    public MessageModel add_product(String product) throws Exception {
        MessageModel messageModel=goodsService.add_product(product);
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/27 16:06
    *商品修改
    */
    @IsLogin
    @RequestMapping("update_product")
    public MessageModel update_product(UpdateProduct product) throws Exception {
        MessageModel messageModel=goodsService.update_product(product);
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/27 16:06
    *商品删除
    */
    @IsLogin
    @RequestMapping("delete_product")
    public MessageModel delete_product(String ids) throws Exception {
        MessageModel messageModel=goodsService.delete_product(ids);
        return messageModel;
    }

    /**
     *@author DY
     *@create 2018/3/27 16:06
     *细节定制商品添加
     */
    @IsLogin
    @RequestMapping("add_product_custom")
    public MessageModel add_product_custom(String customProduct) throws Exception {
        MessageModel messageModel=goodsService.add_product_custom(customProduct);
        return messageModel;
    }

    /**
    *@author DY
    *@create 2018/3/27 16:06
    *细节定制商品修改
    */
    @IsLogin
    @RequestMapping("update_product_custom")
    public MessageModel update_product_custom(ProductCustom productCustom) throws Exception {
        MessageModel messageModel=goodsService.update_product_custom(productCustom);
        return messageModel;
    }
    /**
    *@author DY
    *@create 2018/3/27 16:06
    *细节定制商品删除
    */
    @IsLogin
    @RequestMapping("delete_product_custom")
    public MessageModel delete_product_custom(String ids) throws Exception {
        MessageModel messageModel=goodsService.delete_product_custom(ids);
        return messageModel;
    }


    /**
     *@author DY
     *@create 2018/3/27 16:06
     *商品搜索-商品列表
     */
    @IsLogin
    @RequestMapping("search_goods_list")
    public MessageModel search_goods_list(GoodsDto goodsDto,HttpSession session) throws Exception {
        LoginIdentity loginIdentity = (LoginIdentity)session.getAttribute(Constant.LOGIN_USER_KEY);
        MessageModel messageModel=goodsService.search_goods_list(goodsDto,loginIdentity);
        return messageModel;
    }

    /**
     *@author DY
     *@create 2018/3/27 16:06
     *查看商品回收站
     */
    @IsLogin
    @RequestMapping("goods_recycle_list")
    public MessageModel goods_recycle_list( GoodsDto goodsDto,HttpSession session) throws Exception {
        LoginIdentity loginIdentity = (LoginIdentity)session.getAttribute(Constant.LOGIN_USER_KEY);
        MessageModel messageModel=goodsService.goods_recycle_list(goodsDto,loginIdentity);
        return messageModel;
    }

    /**
     *@author DY
     *@create 2018/3/27 16:06
     *商品回收站-还原-删除
     * 1.还原  2.删除
     */
    @IsLogin
    @RequestMapping("goods_recycle_return")
    public MessageModel goods_recycle_return(String ids,Integer returnOrDelete) throws Exception {
        MessageModel messageModel= goodsService.goods_recycle_return(ids,returnOrDelete);
        return messageModel;
    }

    /**
     *@author DY
     *@create 2018/3/27 16:06
     *商品属性列表
     */
    @IsLogin
    @RequestMapping("goods_attribute_list")
    public MessageModel goods_attribute_list(Integer orderItemId) throws Exception {
        MessageModel messageModel=goodsService.getGoodsAttribute(orderItemId);
        return messageModel;
    }

    /**
     *@author DY
     *@create 2018/3/27 16:06
     *商品尺码添加 --- goods, size , productCategory 所属子品类
     */
    @IsLogin
    @RequestMapping("add_size")
    public MessageModel add_size(Goods goods) throws Exception {
        MessageModel messageModel=goodsService.add_size(goods);
        return messageModel;
    }
    /**
     *@author DY
     *@create 2018/3/27 16:06
     *商品尺码更新
     */
    @IsLogin
    @RequestMapping("update_size")
    public MessageModel update_size(SizeReturn size) throws Exception {
        MessageModel messageModel=goodsService.update_size(size);
        return messageModel;
    }
    /**
     *@author DY
     *@create 2018/3/27 16:06
     *商品尺码添加
     */
    @IsLogin
    @RequestMapping("delete_size")
    public MessageModel delete_size(Integer id) throws Exception {
        MessageModel messageModel=goodsService.delete_size(id);
        return messageModel;
    }

    /**
     *@author DY
     *@create 2018/3/27 16:06
     *商品属性切换--删除
     */
    @IsLogin
    @RequestMapping("goods_product_delete")
    public MessageModel goods_product_delete(Integer goods,Boolean isCustom) throws Exception {
        MessageModel messageModel=goodsService.goods_product_delete(goods,isCustom);
        return messageModel;
    }


}
